# Generated by Django 3.1 on 2021-02-23 15:24
from django.db import migrations, models

import pymess.models.emails


class Migration(migrations.Migration):

    dependencies = [
        ('pymess', '0022_migration'),
    ]

    operations = [
        migrations.AddField(
            model_name='emailmessage',
            name='content_file',
            field=models.FileField(blank=True, null=True, upload_to=pymess.models.emails.generate_content_filename,
                                   verbose_name='content file'),
        ),
        migrations.AlterField(
            model_name='emailmessage',
            name='content',
            field=models.TextField(verbose_name='content', null=True, blank=True)
        ),
        migrations.RenameField(
            model_name='emailmessage',
            old_name='content',
            new_name='old_content',
        ),
        migrations.AlterField(
            model_name='attachment',
            name='file',
            field=models.FileField(upload_to=pymess.models.emails.generate_attachment_filename,
                                   verbose_name='file'),
        ),
    ]



from enum import Enum

from django.conf import settings
from django.utils import timezone
from django.utils.translation import gettext_lazy as _l

from pymess.backend.sms import SMSBackend
from pymess.enums import OutputSMSMessageState

from twilio.rest import TwilioRestClient


class TwilioState(str, Enum):

    ACCEPTED = 'ACCEPTED'
    QUEUED = 'QUEUED'
    SENDING = 'SENDING'
    SENT = 'SENT'
    DELIVERED = 'DELIVERED'
    RECEIVED = 'RECEIVED'
    FAILED = 'FAILED'
    UNDELIVERED = 'UNDELIVERED'


class TwilioSMSBackend(SMSBackend):
    """
    SMS backend implementing twilio service https://www.twilio.com/
    """

    twilio_client = None

    STATES_MAPPING = {
        TwilioState.ACCEPTED: OutputSMSMessageState.SENT,
        TwilioState.QUEUED: OutputSMSMessageState.SENT,
        TwilioState.SENDING: OutputSMSMessageState.SENT,
        TwilioState.SENT: OutputSMSMessageState.SENT,
        TwilioState.DELIVERED: OutputSMSMessageState.DELIVERED,
        TwilioState.RECEIVED: OutputSMSMessageState.DELIVERED,
        TwilioState.FAILED: OutputSMSMessageState.ERROR_UPDATE,
        TwilioState.UNDELIVERED: OutputSMSMessageState.ERROR_UPDATE,
    }

    def _get_twilio_client(self):
        """
        Connect to the twilio service
        """
        if not self.twilio_client:
            self.twilio_client = TwilioRestClient(settings.TWILIO_ACCOUNT_SID, settings.TWILIO_AUTH_TOKEN)
        return self.twilio_client

    def publish_message(self, message):
        """
        Method uses twilio REST client for sending SMS message
        :param message: SMS message
        """
        client = self._get_twilio_client()
        try:
            result = client.messages.create(
                from_=settings.TWILIO_SENDER,
                to=str(message.recipient),
                body=message.content
            )
            self._update_message_after_sending(
                message,
                state=self.STATES_MAPPING[TwilioState(result.status.upper())],
                error=result.error_message if result.error_message else None,
                sent_at=timezone.now()
            )
        except Exception as ex:
            self._update_message_after_sending_error(
                message, error=str(ex)
            )
            # Do not re-raise caught exception. We do not know exact exception to catch so we catch them all
            # and log them into database. Re-raise exception causes transaction rollback (lost of information about
            # exception).



import codecs
import os
from typing import Any

from chamber.shortcuts import change_and_save
from pymess.config import settings

__all__ = [
    "change_and_save",
    "get_email_template_body_from_file",
    "SyncEmailTemplates",
]

def get_email_template_body_from_file(email_template_slug, locale, variant):
    base_dir = settings.EMAIL_HTML_DATA_DIRECTORY
    if locale:
        base_dir = os.path.join(base_dir, locale)
    filename = f'{email_template_slug}_{variant}' if variant else email_template_slug
    file_path = os.path.join(base_dir, f'{filename}.html')
    with codecs.open(file_path, 'r', encoding='utf-8-sig') as file:
        return file.read()



class SyncEmailTemplates:

    def __init__(self, template_slugs=None, locale=None, variant=None, unlock=True):
        self.template_slugs = template_slugs
        self.locale = locale
        self.variant = variant
        self.unlock = unlock
        
    def __call__(self, apps, schema_editor):
        email_template_class = apps.get_model(*settings.EMAIL_TEMPLATE_MODEL.split('.'))
        email_template_qs = email_template_class.objects.all()
        
        filters = {}
        if self.template_slugs:
            filters['slug__in'] = self.template_slugs
        if self.locale:
            filters['locale'] = self.locale
        if self.variant:
            filters['variant'] = self.variant
        
        email_template_qs = email_template_qs.filter(**filters)
        email_templates_found = list(email_template_qs)
        
        if self.template_slugs:
            email_templates_not_found = set(self.template_slugs) - {t.slug for t in email_templates_found}
            assert not email_templates_not_found, f"Email templates {email_templates_not_found} were not found."
        for email_template in email_templates_found:
            email_template.body = get_email_template_body_from_file(email_template.slug, self.locale, self.variant)
            email_template.is_locked = not self.unlock
        email_template_class.objects.bulk_update(email_templates_found, ['body', 'is_locked'])



# -*- coding: utf-8 -*-
# Generated by Django 1.11 on 2018-12-05 08:58
from __future__ import unicode_literals

from django.db import migrations, models
import django.db.models.deletion

from pymess.config import settings


class Migration(migrations.Migration):

    dependencies = [
        ('contenttypes', '0002_remove_content_type_name'),
        ('pymess', '0002_emailmessage_number_of_send_attempts'),
    ]

    operations = [
        migrations.CreateModel(
            name='DialerMessage',
            fields=[
                ('id', models.AutoField(auto_created=True, primary_key=True, serialize=False, verbose_name='ID')),
                ('created_at', models.DateTimeField(auto_now_add=True, db_index=True, verbose_name='created at')),
                ('changed_at', models.DateTimeField(auto_now=True, db_index=True, verbose_name='changed at')),
                ('sent_at', models.DateTimeField(blank=True, editable=False, null=True, verbose_name='sent at')),
                ('recipient', models.CharField(max_length=20, verbose_name='recipient')),
                ('content', models.TextField(verbose_name='content')),
                ('template_slug',
                 models.SlugField(blank=True, editable=False, max_length=100, null=True, verbose_name='slug')),
                ('state', models.IntegerField(
                    choices=[(0, 'not assigned'), (1, 'ready'), (2, 'rescheduled by dialer'), (3, 'call in progress'),
                             (4, 'hangup'), (5, 'done'), (6, 'rescheduled'), (7, 'listened up complete message'),
                             (8, 'listened up partial message'), (9, 'unreachable'), (10, 'declined'),
                             (11, 'unanswered'), (66, 'error'), (77, 'debug'), ], editable=False,
                    verbose_name='state')),
                ('backend',
                 models.CharField(blank=True, editable=False, max_length=250, null=True, verbose_name='backend')),
                ('error', models.TextField(blank=True, editable=False, null=True, verbose_name='error')),
                ('extra_data',
                 models.TextField(blank=True, editable=False, null=True, verbose_name='extra data')),
                ('extra_sender_data',
                 models.TextField(blank=True, editable=False, null=True, verbose_name='extra sender data')),
                ('tag', models.SlugField(blank=True, editable=False, null=True, verbose_name='tag')),
            ],
            options={
                'verbose_name': 'dialer message',
                'verbose_name_plural': 'dialer messages',
                'ordering': ('-created_at',),
                'abstract': False,
            },
        ),
        migrations.CreateModel(
            name='DialerMessageRelatedObject',
            fields=[
                ('id', models.AutoField(auto_created=True, primary_key=True, serialize=False, verbose_name='ID')),
                ('created_at', models.DateTimeField(auto_now_add=True, db_index=True, verbose_name='created at')),
                ('changed_at', models.DateTimeField(auto_now=True, db_index=True, verbose_name='changed at')),
                ('object_id', models.TextField(verbose_name='ID of the related object')),
                ('object_id_int', models.PositiveIntegerField(blank=True, db_index=True, null=True,
                                                              verbose_name='ID of the related object in int format')),
                ('content_type',
                 models.ForeignKey(on_delete=django.db.models.deletion.CASCADE, to='contenttypes.ContentType',
                                   verbose_name='content type of the related object')),
                ('dialer_message',
                 models.ForeignKey(on_delete=django.db.models.deletion.CASCADE, related_name='related_objects',
                                   to='pymess.DialerMessage', verbose_name='dialer message')),
            ],
            options={
                'verbose_name': 'related object of a dialer message',
                'verbose_name_plural': 'related objects of dialer messages',
                'ordering': ('-created_at',),
            },
        ),
        migrations.CreateModel(
            name='DialerTemplate',
            fields=[
                ('created_at', models.DateTimeField(auto_now_add=True, db_index=True, verbose_name='created at')),
                ('changed_at', models.DateTimeField(auto_now=True, db_index=True, verbose_name='changed at')),
                ('slug', models.SlugField(editable=False, max_length=100, primary_key=True, serialize=False,
                                          verbose_name='slug')),
                ('body', models.TextField(null=True, verbose_name='message body')),
            ],
            options={
                'ordering': ('-created_at',),
                'verbose_name': 'dialer template',
                'verbose_name_plural': 'dialer templates',
                'abstract': False,
            },
        ),
        migrations.AddField(
            model_name='dialermessage',
            name='template',
            field=models.ForeignKey(blank=True, null=True, on_delete=django.db.models.deletion.SET_NULL,
                                    related_name='dialer_messages', to=settings.DIALER_TEMPLATE_MODEL,
                                    verbose_name='template'),
        ),
    ]



import os
import base64

import requests

from json.decoder import JSONDecodeError

from enum import Enum

from django.utils import timezone
from django.utils.translation import gettext_lazy as _
from django.utils.translation import gettext

import mandrill

from pymess.backend.emails import EmailBackend
from pymess.enums import EmailMessageState
from pymess.config import settings
from pymess.utils.logged_requests import generate_session


class MandrillState(str, Enum):

    SENT = 'SENT'
    QUEUED = 'QUEUED'
    SCHEDULED = 'SCHEDULED'
    REJECTED = 'REJECTED'
    INVALID = 'INVALID'


class MandrillEmailBackend(EmailBackend):
    """
    E-mail backend implementing Mandrill service (https://mandrillapp.com/api/docs/index.python.html).
    """

    MANDRILL_STATES_MAPPING = {
        MandrillState.SENT: EmailMessageState.SENT,
        MandrillState.QUEUED: EmailMessageState.SENT,
        MandrillState.SCHEDULED: EmailMessageState.SENT,
        MandrillState.REJECTED: EmailMessageState.ERROR,
        MandrillState.INVALID: EmailMessageState.ERROR,
    }

    config = {
        'HEADERS': None,
        'TRACK_OPENS': False,
        'TRACK_CLICKS': False,
        'AUTO_TEXT': False,
        'INLINE_CSS': False,
        'URL_STRIP_QS': False,
        'PRESERVE_RECIPIENTS': False,
        'VIEW_CONTENT_LINK': True,
        'ASYNC': False,
        'TIMEOUT': 5,  # 5s
    }

    def _serialize_attachments(self, message):
        return [
            {
                'type': attachment.content_type,
                'name': attachment.filename or os.path.basename(attachment.file.name),
                'content': base64.b64encode(attachment.file.read()).decode('utf-8')
            } for attachment in message.attachments.all()
        ]

    def _create_client(self, message):
        mandrill_client = mandrill.Mandrill(self.config['KEY'])
        mandrill_client.session = generate_session(
            slug='pymess - Mandrill',
            related_objects=(message,),
            timeout=self.config['TIMEOUT']
        )
        return mandrill_client

    def publish_message(self, message):
        mandrill_client = self._create_client(message)
        try:
            result = mandrill_client.messages.send(
                message={
                    'to': [{'email': message.recipient}],
                    'from_email': message.sender,
                    'from_name': message.sender_name,
                    'html': message.content,
                    'subject': message.subject,
                    'headers': self.config['HEADERS'],
                    'track_opens': self.config['TRACK_OPENS'],
                    'auto_text': self.config['AUTO_TEXT'],
                    'inline_css': self.config['INLINE_CSS'],
                    'url_strip_qs': self.config['URL_STRIP_QS'],
                    'preserve_recipients': self.config['PRESERVE_RECIPIENTS'],
                    'view_content_link': self.config['VIEW_CONTENT_LINK'],
                    'async': self.config['ASYNC'],
                    'attachments': self._serialize_attachments(message)
                },
            )[0]
            mandrill_state = MandrillState(result['status'].upper())
            state = self.MANDRILL_STATES_MAPPING.get(mandrill_state)
            error = None
            if mandrill_state == MandrillState.INVALID:
                error = gettext('invalid')
            elif mandrill_state == MandrillState.REJECTED:
                error = gettext('rejected, mandrill message: "{}"').format(result['reject_reason'])

            extra_sender_data = message.extra_sender_data or {}
            extra_sender_data['result'] = result
            self._update_message_after_sending(
                message,
                state=state,
                sent_at=timezone.now(),
                extra_sender_data=extra_sender_data,
                error=error,
                external_id=result.get('_id')
            )
        except (mandrill.Error, JSONDecodeError, requests.exceptions.RequestException) as ex:
            self._update_message_after_sending_error(
                message,
                error=str(ex)
            )
            # Do not re-raise caught exception. Re-raise exception causes transaction rollback (lost of information
            # about exception).

    def pull_message_info(self, message):
        if message.external_id:
            mandrill_client = self._create_client(message)
            try:
                info = mandrill_client.messages.info(message.external_id)
                self._update_message(
                    message,
                    extra_sender_data={**message.extra_sender_data, 'info': info},
                    info_changed_at=timezone.now(),
                    update_only_changed_fields=True,
                )
            except mandrill.UnknownMessageError:
                self._update_message(message, info_changed_at=timezone.now(), update_only_changed_fields=True)
        else:
            self._update_message(message, info_changed_at=timezone.now(), update_only_changed_fields=True)



from django.db import migrations, models


class Migration(migrations.Migration):

    dependencies = [
        ('pymess', '0027_migration'),
    ]

    operations = [
        migrations.AddField(
            model_name='smstemplate',
            name='is_secret',
            field=models.BooleanField(default=False),
            preserve_default=False,
        ),
        migrations.AlterField(
            model_name='smstemplate',
            name='is_secret',
            field=models.BooleanField(default=True),
        ),
    ]



# Generated by Django 3.1 on 2021-12-31 10:25

from django.db import migrations, models


class Migration(migrations.Migration):

    dependencies = [
        ('pymess', '0026_migration'),
    ]

    operations = [
        migrations.AddField(
            model_name='pushnotificationtemplate',
            name='redirect_url',
            field=models.TextField(blank=True, null=True, verbose_name='redirect URL'),
        ),
        migrations.AddField(
            model_name='pushnotificationmessage',
            name='redirect_url',
            field=models.TextField(blank=True, null=True, verbose_name='redirect URL'),
        ),
    ]



# -*- coding: utf-8 -*-
# Generated by Django 1.11.26 on 2020-02-25 19:11
from __future__ import unicode_literals

from django.db import migrations, models


class Migration(migrations.Migration):
    dependencies = [
        ('pymess', '0015_auto_20200110_0904'),
    ]

    operations = [
        migrations.RemoveField(
            model_name='dialermessagerelatedobject',
            name='object_id_int',
        ),
        migrations.RemoveField(
            model_name='dialertemplatedisallowedobject',
            name='object_id_int',
        ),
        migrations.RemoveField(
            model_name='emailrelatedobject',
            name='object_id_int',
        ),
        migrations.RemoveField(
            model_name='emailtemplatedisallowedobject',
            name='object_id_int',
        ),
        migrations.RemoveField(
            model_name='outputsmsrelatedobject',
            name='object_id_int',
        ),
        migrations.RemoveField(
            model_name='pushnotificationmessagerelatedobject',
            name='object_id_int',
        ),
        migrations.RemoveField(
            model_name='smstemplatedisallowedobject',
            name='object_id_int',
        ),
        migrations.AddField(
            model_name='dialermessage',
            name='number_of_send_attempts',
            field=models.PositiveIntegerField(default=0, verbose_name='number of send attempts'),
        ),
        migrations.AddField(
            model_name='outputsmsmessage',
            name='number_of_send_attempts',
            field=models.PositiveIntegerField(default=0, verbose_name='number of send attempts'),
        ),
        migrations.AddField(
            model_name='pushnotificationmessage',
            name='number_of_send_attempts',
            field=models.PositiveIntegerField(default=0, verbose_name='number of send attempts'),
        ),
        migrations.AlterField(
            model_name='dialermessage',
            name='recipient',
            field=models.CharField(db_index=True, max_length=20, verbose_name='recipient'),
        ),
        migrations.AlterField(
            model_name='dialermessage',
            name='state',
            field=models.IntegerField(
                choices=[(-1, 'waiting'), (0, 'not assigned'), (1, 'ready'), (2, 'rescheduled by dialer'),
                         (3, 'call in progress'), (4, 'hangup'), (5, 'done'), (6, 'rescheduled'),
                         (7, 'listened up complete message'), (8, 'listened up partial message'), (9, 'unreachable'),
                         (10, 'declined'), (11, 'unanswered'), (12, 'unanswered - hangup by dialer'),
                         (13, 'answered - hangup by customer'), (66, 'error'), (77, 'debug'),
                         (88, 'error essage was not sent')], editable=False, verbose_name='state'),
        ),
        migrations.AlterField(
            model_name='dialermessagerelatedobject',
            name='object_id',
            field=models.TextField(db_index=True, verbose_name='ID of the related object'),
        ),
        migrations.AlterField(
            model_name='dialertemplatedisallowedobject',
            name='object_id',
            field=models.TextField(db_index=True, verbose_name='ID of the related object'),
        ),
        migrations.AlterField(
            model_name='emailmessage',
            name='recipient',
            field=models.EmailField(db_index=True, max_length=254, verbose_name='recipient'),
        ),
        migrations.AlterField(
            model_name='emailmessage',
            name='state',
            field=models.IntegerField(
                choices=[(1, 'waiting'), (2, 'sending'), (3, 'sent'), (4, 'error essage was not sent'), (5, 'debug')],
                editable=False, verbose_name='state'),
        ),
        migrations.AlterField(
            model_name='emailrelatedobject',
            name='object_id',
            field=models.TextField(db_index=True, verbose_name='ID of the related object'),
        ),
        migrations.AlterField(
            model_name='emailtemplatedisallowedobject',
            name='object_id',
            field=models.TextField(db_index=True, verbose_name='ID of the related object'),
        ),
        migrations.AlterField(
            model_name='outputsmsmessage',
            name='recipient',
            field=models.CharField(db_index=True, max_length=20, verbose_name='recipient'),
        ),
        migrations.AlterField(
            model_name='outputsmsmessage',
            name='state',
            field=models.IntegerField(
                choices=[(1, 'waiting'), (2, 'unknown'), (3, 'sending'), (4, 'sent'), (5, 'error'), (6, 'debug'),
                         (7, 'delivered'), (8, 'error message was not sent')], editable=False, verbose_name='state'),
        ),
        migrations.AlterField(
            model_name='outputsmsrelatedobject',
            name='object_id',
            field=models.TextField(db_index=True, verbose_name='ID of the related object'),
        ),
        migrations.AlterField(
            model_name='pushnotificationmessage',
            name='recipient',
            field=models.CharField(db_index=True, max_length=20, verbose_name='recipient'),
        ),
        migrations.AlterField(
            model_name='pushnotificationmessage',
            name='state',
            field=models.PositiveIntegerField(
                choices=[(1, 'waiting'), (2, 'sent'), (3, 'error essage was not sent'), (4, 'debug')], editable=False,
                verbose_name='state'),
        ),
        migrations.AlterField(
            model_name='pushnotificationmessagerelatedobject',
            name='object_id',
            field=models.TextField(db_index=True, verbose_name='ID of the related object'),
        ),
        migrations.AlterField(
            model_name='smstemplatedisallowedobject',
            name='object_id',
            field=models.TextField(db_index=True, verbose_name='ID of the related object'),
        ),
    ]
